<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       www.javmah.tk
 * @since      1.0.0
 *
 * @package    Woo_Orders_To_Google_Spreadsheet
 * @subpackage Woo_Orders_To_Google_Spreadsheet/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Woo_Orders_To_Google_Spreadsheet
 * @subpackage Woo_Orders_To_Google_Spreadsheet/admin
 * @author     javmah <jaedmah@gmail.com>
 */
class Woo_Orders_To_Google_Spreadsheet_Admin {

	private $plugin_name;

	private $version;
	

	private $column_titles = array(
		"A"=>"Order ID",
		"B"=>"Date",
	 	"C"=>"Order status",
	 	"D"=>"Phone number",
	 	"E"=>"Email Address ",
		"F"=>"Customer note",
		"G"=>"Billing address",
		"H"=>"Shipping address",
		"I"=>"Payment methode",
		"J"=>"Shipping methode",
		"K"=>"Order items",
		"L"=>"Currency",
		"M"=>"Shipping total",
		"N"=>"Order total",
		"O"=>"IP Address"
	);

	public function __construct( $plugin_name, $version ) {
		$this->plugin_name = $plugin_name;
		$this->version = $version;
	}

	public function enqueue_styles() {
		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/woo-orders-to-google-spreadsheet-admin.css', array(), $this->version, 'all' );
	}

	public function enqueue_scripts() {
		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/woo-orders-to-google-spreadsheet-admin.js', array( 'jquery' ), $this->version, false );
	}

	# Admin menu init
	public function woogs_admin_menu($value=''){
		add_menu_page( __( 'Woogs','woo-orders-to-google-spreadsheet' ), __( 'Woogs','woo-orders-to-google-spreadsheet'),'manage_options','woo-orders-to-google-spreadsheet', array( $this,'woogs_view_func' ),'dashicons-media-spreadsheet');
	}

	# woogs views
	public function woogs_view_func( ){
		//  generating access token if it is not Valid 
		$this->woogs_get_access_token_by_refresh_token( get_option( "woogs_access_token" ) );
		// Including External view File from Partial Folder 
		require_once plugin_dir_path(dirname(__FILE__)).'admin/partials/woo-orders-to-google-spreadsheet-admin-display.php';
	}

	# Admin notice for Testing 
	public function woogs_admin_notice($value=''){
		//  generating access token if it is not Valid 
		// $this->woogs_get_access_token_by_refresh_token( get_option( "woogs_access_token" ) );
	}

	# Handaling Form Submission || Save googele code
	public function woogs_google_code($value=''){
		$nonce 						= sanitize_text_field( $_POST['woogs_nonce'] );
		$woogs_authorization_code 	= sanitize_text_field( $_POST['woogs_authorization_code'] );

		# Empty Cheack 
		if (empty($nonce) || empty($woogs_authorization_code) ) {
			wp_redirect( 'admin.php?page=woo-orders-to-google-spreadsheet&msg='.urlencode('Access Code is Empty or nonce is not Valid') );
			die( __( 'Access Code is Empty ', 'Wpgcs' ) );
		}

		# Now Get The Refresh Token and Access Token 
		$client_id 		= '1010705240979-jol43e06lb2r8pm3r0nr6f80moma8sur.apps.googleusercontent.com';
		$client_secret 	= 'udWXl9BFtgnudOGPl5MYAxYx';
		$redirect 		= 'urn:ietf:wg:oauth:2.0:oob';
		$refresh_token 	= $this->woogs_get_refresh_token_by_authorization_code($client_id , $client_secret , $woogs_authorization_code );
		
		if ( $refresh_token['response_code'] == 200 ) {
			# Success
			update_option( "woogs_access_token ", json_encode( $refresh_token , true ) );
			wp_redirect( 'admin.php?page=woo-orders-to-google-spreadsheet&msg=success' );
			exit;
		}else{
			#error 
			wp_redirect( 'admin.php?page=woo-orders-to-google-spreadsheet&msg='.urlencode('client_id or client_secret is empty') );
			exit;
		}
	}

	# Handaling Form Submission || This Function Will handale Settings Form
	public function woogs_settings(){
		$nonce = wp_create_nonce('woogs_nonce');
		$token =  json_decode(get_option( "woogs_access_token" ), true);

		if( !isset($_POST['woogs_nonce'] ) || $_POST['woogs_nonce']  != $nonce   ){
			// return "error Nonce";
			wp_redirect( 'admin.php?page=woo-orders-to-google-spreadsheet&msg='.urlencode('Error Nonce') );
			exit;
		}

		if( !isset($_POST['spreadsheet'] ) ||  empty($_POST['spreadsheet'])   ){
			// return "error no spreadsheet";
			wp_redirect( 'admin.php?page=woo-orders-to-google-spreadsheet&msg='.urlencode('Error no spreadsheet') );
			exit;
		}

		if( !isset($_POST['worksheet'] ) ||  empty($_POST['worksheet'])   ){
			// return "error no worksheet";
			wp_redirect( 'admin.php?page=woo-orders-to-google-spreadsheet&msg='.urlencode('Error no worksheet') );
			exit;
		}

		// Submitted Data from Settings Form
		$settings_form_data = array();
		$settings_form_data['append_row_after'] = sanitize_text_field($_POST['append_row_after'] );
		$settings_form_data['spreadsheet'] 		= sanitize_text_field($_POST['spreadsheet'] );
		$settings_form_data['worksheet'] 		= sanitize_text_field($_POST['worksheet'] );

		//  JSON encode and Update to wp option table
		update_option( "woogs_settings", json_encode( $settings_form_data , true )  );

		// column titles name set if Cheacked.
		if ( isset( $_POST['create_column_title'] )  AND  $_POST['create_column_title'] == 'y' ) {

			if (isset( $token['access_token'] ) AND !empty(  $token['access_token'] )  ) {
				$this->woogs_append_new_row( sanitize_text_field( $token['access_token']) , sanitize_text_field($_POST['spreadsheet']) , sanitize_text_field($_POST['worksheet']) ,  $this->column_titles );
			}		
		}

		// Redirect
		wp_redirect( 'admin.php?page=woo-orders-to-google-spreadsheet&msg=settings_saved');
		exit;
	}

	# Gatting Google Users  Authorization Code to Refresh token 
	public  function woogs_get_refresh_token_by_authorization_code( $client_id = '', $client_secret = '', $authorization_code = '' ) {
		$response = array();
		// Empty client id or client secret
		if ( empty( $client_id ) || empty( $client_secret ) ) {
			wp_redirect( 'admin.php?page=woo-orders-to-google-spreadsheet&msg='.urlencode('client_id or client_secret is empty') );
			exit;
		}
		// Empty authorization code
		if ( empty( $authorization_code ) ) {
			wp_redirect( 'admin.php?page=woo-orders-to-google-spreadsheet&msg='.urlencode('authorization code is empty') );
			exit;
		}

		$url ='https://accounts.google.com/o/oauth2/token';
		$args = Array(
		    'headers' => Array(),
		    'body'    => array(
				'code'          =>$authorization_code,
				'client_id'     =>$client_id,
				'client_secret' =>$client_secret,
				'redirect_uri'  =>'urn:ietf:wg:oauth:2.0:oob',
				'grant_type'    =>'authorization_code',
				'access_type'   =>'offline',
			)
		);

		$google_return   =  wp_remote_post( $url, $args );
		$google_body     =  json_decode( $google_return['body'], true );

		if ( $google_return['response']['code'] == 200 ) {
			$response['authorization_code'] = $authorization_code;
			$response['access_token']     	= $google_body['access_token'];
			$response['expires_in']       	= $google_body['expires_in'];
			$response['refresh_token']    	= $google_body['refresh_token'];
			$response['scope']            	= $google_body['scope'];
			$response['token_type']       	= $google_body['token_type'];
			$response['created']          	= time();
			$response['expire']           	= time() + $google_body['expires_in'];
			$response['response_code']    	= $google_return['response']['code'];
			$response['response_message'] 	= $google_return['response']['message'];
		} else {
			// error
			$response['authorization_code'] = $authorization_code;
			$response['access_token']     	= '';
			$response['expires_in']       	= '';
			$response['refresh_token']    	= '';
			$response['scope']            	= '';
			$response['token_type']       	= '';
			$response['created']          	= time();
			$response['expire']           	= '';
			$response['response_code']    	= $google_return['response']['code'];
			$response['response_message'] 	= $google_return['response']['message'];
		}

		return $response ;
	}

	# Gatting new Access token by refresh token 
	public  function woogs_get_access_token_by_refresh_token( $token_json =''){

		$client_id 		= '1010705240979-jol43e06lb2r8pm3r0nr6f80moma8sur.apps.googleusercontent.com';
		$client_secret 	= 'udWXl9BFtgnudOGPl5MYAxYx';

		$token_array 	= json_decode($token_json , true);
		$response 		= array();

		if ( empty($token_json) || empty($token_array['refresh_token'])  ) {
			return false ; 
		}

		// Token time is Exist and Time is Valid || so no need to genarate a New token 
		if ( empty($token_array['expire']) ||  $token_array['expire'] > time()  ) {
			return false ; 
		}

		// Token time is Expire So Genarate a New Token 
		$url ='https://www.googleapis.com/oauth2/v4/token';
		$args = Array(
		    'headers' => Array(),
		    'body' => array(
				'refresh_token'	=>	$token_array['refresh_token'] ,
				'client_id'		=>	$client_id ,
				'client_secret'	=>	$client_secret ,
				'grant_type'	=>	'refresh_token',
			)
		);

		$google_return  = wp_remote_post($url, $args );
		
		// Body Encode to JSON
		if ($google_return['response']['code'] == 200) {
			$google_body  	=  json_decode($google_return['body'], true);
			
			$response['created']        	= time();
			$response['authorization_code']	= $token_array['authorization_code'];
			$response['refresh_token']		= $token_array['refresh_token'];
			$response['access_token']		= $google_body['access_token'];
			$response['expires_in']			= $google_body['expires_in'];
			$response['expire'] 			= time() + $google_body['expires_in'];
		} else {
			# error
			# copy old token and retern it 
			$response = $token_array ;
		}

		# if differant array save it to option 
		if ($response != $token_array) {
			update_option( "woogs_access_token ", json_encode( $response , true )  );
			return true ; 
		}

		return false ;
	}

	# Saved token validation cheack
	public function access_token_validation_cheack($token=''){
		if (empty( $token )) {
			return false;
		}

		$url =  'https://www.googleapis.com/oauth2/v1/tokeninfo?access_token='.$token  ;
		$token_data   =  wp_remote_get( $url );
		return $token_data['body']  ;
	}

	# Getting the spreadsheets from google server 
	public function spreadsheets($token=''){
		$google_spreadsheets_return = wp_remote_get("https://www.googleapis.com/drive/v3/files?q=mimeType%20%3D%20'application%2Fvnd.google-apps.spreadsheet'
		     &access_token=".$token['access_token'] );

		if($google_spreadsheets_return['response']['code'] != 200){
			return array(false , $google_spreadsheets_return['response']['message'] );
		}

		$body = json_decode( $google_spreadsheets_return['body'] , true );
		$spreadsheet_list = $body['files'] ;
		$spreadsheets_id_and_title = array();

		foreach($spreadsheet_list  as $spreadsheet ){
			$spreadsheets_id_and_title[$spreadsheet['id']] = $spreadsheet['name'] ;
		}

		return array( true , $spreadsheets_id_and_title );
	}

	# AJAX worksheet load from a Spacific Spreed Sheet
	public function woogs_GetWorksheetsAspreadsheet($value=''){
		$spreadsheetId 		= sanitize_text_field( $_POST['spreadsheetId'] );
		$fts_access_token 	= json_decode(get_option('woogs_access_token'), true);

		if ($spreadsheetId) {
			echo json_encode($this->worksheets($fts_access_token,$spreadsheetId));
		}
		exit();
	}

	# Gatting worksheet of a Spreed Sheet by Spreedsheet Id
	public  function worksheets($token='' , $spreadsheetId=''){
		if (empty($token)) return false;
		if (empty($spreadsheetId)) return false;

		// Cheack access Token Validation 
		// $this->access_token_validation_cheack($token);

		$google_return = wp_remote_get('https://sheets.googleapis.com/v4/spreadsheets/'.$spreadsheetId.'/?access_token='.$token['access_token']);
		$body = json_decode( $google_return['body'] , true);

		$sheets = array();
		foreach ($body['sheets'] as $value) {
			$sheets[$value['properties']['sheetId']] = $value['properties']['title'] ;
		}

		return $sheets ;
	}

	#***********************************************************************************************
	#									Insert data into spreadsheets
	#***********************************************************************************************

	public  function woogs_append_new_row($access_token='',$spreadsheet_id='',$worksheet_name='' ,$data_array='' ){
		
		if (empty( $access_token ) || empty( $spreadsheet_id ) ) {
			return "access_token or  spreadsheet_id is empty";
		}

		if (empty( $worksheet_name ) || empty( $data_array ) ) {
			return "worksheet_name or  data_array is empty";
		}

		# Array Convartion Starts 
		$final = array();
		foreach($data_array as $key => $val){
		  	if($val){
		   		$final[] = $val ;
		  	}else{
		    	$final[] ="--" ;
		  	}
		}

		// Gatting Last Key  of The Passed Array
		end($data_array);       
		$last_key = key($data_array); 

		# Array Convartion Enda

		$url ='https://sheets.googleapis.com/v4/spreadsheets/'.$spreadsheet_id.'/values/'.$worksheet_name.'!A:'.$last_key.':append?valueInputOption=USER_ENTERED';
		$args = Array(
		    'headers' => Array(
		    	'Authorization'=>'Bearer '. $access_token,
		    	'Content-Type'=> 'application/json'
		    ),

		    'body' => '{"range":"'.$worksheet_name.'!A:'.$last_key.'","majorDimension":"ROWS","values":['.json_encode($final).']}'
		);

		$google_return   = wp_remote_post($url, $args );

		if ( $google_return['response']['code'] == 200) {
			return array(1);
		}else{
			return array(0, $google_return);
		}
	}

	
	#***********************************************************************************************
	#									Insert Data by event functions
	#***********************************************************************************************

	# insert a row depands on order status 
	public function woogs_append_row_after_woocom_action( $order_id, $this_status_transition_from, $this_status_transition_to){
		// generating access token if it is not Valid 
		$this->woogs_get_access_token_by_refresh_token( get_option( "woogs_access_token" ) );
		// 

		$token =  json_decode(get_option( "woogs_access_token" ), true);
		$settings =  json_decode(get_option( "woogs_settings" ), true);

		if ( empty($settings['append_row_after'])  || $settings['append_row_after'] == 'new_order' ) {
			return;
		}

		if ( $settings['append_row_after'] ==  $this_status_transition_to ) {
			# Getting Order information Starts 
			$order_data = $this->woogs_get_woocommerce_order_info($order_id); 
			if ($order_data['status']) {
				unset($order_data['status']);
				$this->woogs_append_new_row( $token['access_token'] ,$settings['spreadsheet'] , $settings['worksheet'] , $order_data ); 
			}
			# Getting Order information Ends
		}

		# if changed to every status Change
		if ($settings['append_row_after'] =='status_change') {
			# Getting Order information Starts 
			$order_data = $this->woogs_get_woocommerce_order_info($order_id); 
			if ($order_data['status']) {
				unset($order_data['status']);
				$ret_back = $this->woogs_append_new_row( $token['access_token'] ,$settings['spreadsheet'] , $settings['worksheet'] , $order_data ); 
			}
			# Getting Order information Ends
		}

	}

	# Create a trell card if action selected in every order
	public function woogs_woocommerce_new_order_action( $order_id ){
		//  generating access token if it is not Valid 
		$this->woogs_get_access_token_by_refresh_token( get_option( "woogs_access_token" ) );
		// 

		$settings =  json_decode(get_option( "woogs_settings" ), true);
		if ( empty($settings['append_row_after'])  || $settings['append_row_after'] != 'new_order' ) {
			return ;
		}

		$token =  json_decode(get_option( "woogs_access_token" ), true);
		# Getting Order information Starts 
		$order_data = $this->woogs_get_woocommerce_order_info($order_id); 
		if ($order_data['status']) {
			unset($order_data['status']);
			$this->woogs_append_new_row( $token['access_token'] ,$settings['spreadsheet'] , $settings['worksheet'] , $order_data ); 
		}
		# Getting Order information Ends
		return ;
	}

	
	# Getting specific order information of a user by order ID
	public function woogs_get_woocommerce_order_info($order_id=''){
		
		$order = wc_get_order($order_id); 
		$products = array();

		if ($order) {
			$i = 0;
			$products['status'] = 1 ;

			// order ID
			$products['A'] = $order->get_id();

			// Date 
			$products['B'] = date( "F j, Y, g:i a", strtotime( $order->get_date_created() )); 

			// order status
			$products['C'] = $order->get_status();  
			
			// Phone Number 
			$products['D'] =  $order->get_billing_phone(); 

			// Order total
			// $products['E'] = $order->get_billing_email();
			$products['E'] = '--';

			// customer note
			$products['F'] =  $order->get_customer_note();

			// billing address;
			$products['G'] = $order->get_billing_first_name()." ".$order->get_billing_last_name()." , ".$order->get_billing_address_1()." , ".$order->get_billing_address_2()." , ".$order->get_billing_city() ." , ".$order->get_billing_country();

			// shipping_address
			$products['H'] = $order->get_shipping_first_name()." ".$order->get_shipping_last_name()." , ".$order->get_shipping_address_1()." , ".$order->get_shipping_address_2()." , ".$order->get_shipping_city() ." , ".$order->get_shipping_country();

			// payment method title
			$products['I'] = $order->get_payment_method_title();

			// shipping method
			// $products['J'] = $order->get_shipping_method();
			$products['J'] = '--';
			
			// Order items
			$products['K'] ='';
			
			// $products['date'] = $order->get_date_created();
			foreach ($order->get_items() as $item_id => $item_data) {
				$i ++ ; 
			    $product = $item_data->get_product();
			    $product_name = $product->get_name(); 			// Get the product name
			    $item_quantity = $item_data->get_quantity();	// Get the item quantity
			    $item_total = $item_data->get_total(); 			// Get the item line total
			    // products_list
			    $products['K']  .= "\n". $i.". ".$product_name.' | Qty : '.$item_quantity.' | Item total: '. number_format( $item_total, 2 );
			}

			// Currency
			$products['L'] = $order->get_currency();

			// shipping total
			// $products['M'] = $order->get_shipping_total();
			$products['M'] = '--';

			// Order total
			// $products['N'] = $order->get_total();
			$products['N'] = '--';

			// IP address
			$products['O'] = $order->get_customer_ip_address();
		}else{
			$products['status'] = 0 ;
			$products['products_list'] = array();
			$products['customer_note'] = "";
			$products['billing_address'] = "";
			$products['shipping_address'] = "";
		}
		return $products ;
	}

}


#..............................................................................................................
# Please let me Know if you need any kind of Help .
# Bast Regards & Thnak you .
# javmah [ jaedmah@gmail.com ]