<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       www.javmah.tk
 * @since      1.0.0
 *
 * @package    Woo_Orders_To_Google_Spreadsheet
 * @subpackage Woo_Orders_To_Google_Spreadsheet/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Woo_Orders_To_Google_Spreadsheet
 * @subpackage Woo_Orders_To_Google_Spreadsheet/includes
 * @author     javmah <jaedmah@gmail.com>
 */
class Woo_Orders_To_Google_Spreadsheet {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Woo_Orders_To_Google_Spreadsheet_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'WOO_ORDERS_TO_GOOGLE_SPREADSHEET_VERSION' ) ) {
			$this->version = WOO_ORDERS_TO_GOOGLE_SPREADSHEET_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'woo-orders-to-google-spreadsheet';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Woo_Orders_To_Google_Spreadsheet_Loader. Orchestrates the hooks of the plugin.
	 * - Woo_Orders_To_Google_Spreadsheet_i18n. Defines internationalization functionality.
	 * - Woo_Orders_To_Google_Spreadsheet_Admin. Defines all hooks for the admin area.
	 * - Woo_Orders_To_Google_Spreadsheet_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-woo-orders-to-google-spreadsheet-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-woo-orders-to-google-spreadsheet-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-woo-orders-to-google-spreadsheet-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-woo-orders-to-google-spreadsheet-public.php';

		$this->loader = new Woo_Orders_To_Google_Spreadsheet_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Woo_Orders_To_Google_Spreadsheet_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Woo_Orders_To_Google_Spreadsheet_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Woo_Orders_To_Google_Spreadsheet_Admin( $this->get_plugin_name(), $this->get_version() );

		# enqueue scripts and styles
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
		# Admin Menu 
		$this->loader->add_action( 'admin_menu', $plugin_admin, 'woogs_admin_menu' );
		# Admin Notice
		$this->loader->add_action( 'admin_notices', $plugin_admin, 'woogs_admin_notice' );
		# Settings Form Submission 
		$this->loader->add_action( 'admin_post_woogs_gcode', $plugin_admin,'woogs_google_code' );
		$this->loader->add_action( 'admin_post_nopriv_woogs_gcode', $plugin_admin, 'woogs_google_code' );
		# Settings 
		$this->loader->add_action( 'admin_post_woogs_settings', $plugin_admin,'woogs_settings' );
		$this->loader->add_action( 'admin_post_nopriv_woogs_settings', $plugin_admin, 'woogs_settings' );
		# Wp AJAX 
		$this->loader->add_action('wp_ajax_woogs_action', $plugin_admin,'woogs_GetWorksheetsAspreadsheet');
		$this->loader->add_action('wp_ajax_nopriv_woogs_action', $plugin_admin,'woogs_GetWorksheetsAspreadsheet');
		# Woocommerce Order Status Changed
		$this->loader->add_action( 'woocommerce_order_status_changed',$plugin_admin,'woogs_append_row_after_woocom_action',10,3);
		# WooCommerce New Order
		$this->loader->add_action( 'woocommerce_thankyou', $plugin_admin, 'woogs_woocommerce_new_order_action',10, 1);

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Woo_Orders_To_Google_Spreadsheet_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Woo_Orders_To_Google_Spreadsheet_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
